(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.toast = {}));
}(this, (function (exports) { 'use strict';

  var waitFor = function (ms) { return new Promise(function (resolve) { return setTimeout(resolve, ms); }); };

  var instances = new Set();
  var container;
  var Toast = function Toast(message, options) {
    if ( options === void 0 ) options = {};

    var timeout = options.timeout; if ( timeout === void 0 ) timeout = 0;
    var action = options.action;
    var type = options.type; if ( type === void 0 ) type = 'default';
    var cancel = options.cancel;
    this.message = message;
    this.options = {
      timeout: timeout,
      action: action,
      type: type,
      cancel: cancel
    };
    this.setContainer();
    this.insert();
    instances.add(this);
  };

  Toast.prototype.insert = function insert () {
      var this$1 = this;

    var el = document.createElement('div');
    el.className = 'toast';
    el.setAttribute('aria-live', 'assertive');
    el.setAttribute('aria-atomic', 'true');
    el.setAttribute('aria-hidden', 'false');
    var ref = this.options;
      var action = ref.action;
      var type = ref.type;
      var cancel = ref.cancel;
    var inner = document.createElement('div');
    inner.className = 'toast-inner';
    var text = document.createElement('span');
    text.className = 'toast-text';
    inner.classList.add(type);
    text.textContent = this.message;
    inner.appendChild(text);

    if (cancel) {
      var button = document.createElement('button');
      button.className = 'toast-button cancel-button';
      button.textContent = cancel;
      button.type = 'text';

      button.onclick = function () { return this$1.destory(); };

      inner.appendChild(button);
    }

    if (action) {
      var button$1 = document.createElement('button');
      button$1.className = 'toast-button';
      button$1.textContent = action.text;
      button$1.type = 'text';

      button$1.onclick = function () {
        this$1.stopTimer();

        if (action.callback) {
          action.callback(this$1);
        } else {
          this$1.destory();
        }
      };

      inner.appendChild(button$1);
    }

    el.appendChild(inner);
    this.startTimer();
    this.el = el;
    container.appendChild(el); // Delay to set slide-up transition

    waitFor(50).then(sortToast);
  };

  Toast.prototype.destory = function destory () {
    var ref = this;
      var el = ref.el;
    if (!el) { return; }
    container.removeChild(el);
    instances.delete(this);
    sortToast();
  };

  Toast.prototype.setContainer = function setContainer () {
    container = document.querySelector('.toast-container');

    if (!container) {
      container = document.createElement('div');
      container.className = 'toast-container';
      document.body.appendChild(container);
    } // Stop all instance timer when mouse enter


    container.addEventListener('mouseenter', function () {
      instances.forEach(function (instance) { return instance.stopTimer(); });
    }); // Restart all instance timer when mouse leave

    container.addEventListener('mouseleave', function () {
      instances.forEach(function (instance) { return instance.startTimer(); });
    });
  };

  Toast.prototype.startTimer = function startTimer () {
      var this$1 = this;

    if (this.options.timeout && !this.timeoutId) {
      this.timeoutId = self.setTimeout(function () { return this$1.destory(); }, this.options.timeout);
    }
  };

  Toast.prototype.stopTimer = function stopTimer () {
    if (this.timeoutId) {
      clearTimeout(this.timeoutId);
      this.timeoutId = undefined;
    }
  };
  function createToast(message, options) {
    return new Toast(message, options);
  }
  function destoryAllToasts() {
    if (!container) { return; }
    instances.clear();

    while (container.firstChild) {
      container.removeChild(container.firstChild);
    }
  }

  function sortToast() {
    var toasts = Array.from(instances).reverse().slice(0, 4);
    var heights = [];
    toasts.forEach(function (toast, index) {
      var sortIndex = index + 1;
      var el = toast.el;
      var height = +(el.getAttribute('data-height') || 0) || el.clientHeight;
      heights.push(height);
      el.className = "toast toast-" + sortIndex;
      el.dataset.height = '' + height;
      el.style.setProperty('--index', '' + sortIndex);
      el.style.setProperty('--height', height + 'px');
      el.style.setProperty('--front-height', ((heights[0]) + "px"));

      if (sortIndex > 1) {
        var hoverOffsetY = heights.slice(0, sortIndex - 1).reduce(function (res, next) { return res += next; }, 0);
        el.style.setProperty('--hover-offset-y', ("-" + hoverOffsetY + "px"));
      } else {
        el.style.removeProperty('--hover-offset-y');
      }
    });
  }

  exports.Toast = Toast;
  exports.createToast = createToast;
  exports.destoryAllToasts = destoryAllToasts;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
